@@ -25,7 +25,9 @@
 #include "config.h"
 #include <gcu/application.h>
 #include <gcu/atom.h>
+#include <gcu/bond.h>
 #include <gcu/document.h>
+#include <gcu/element.h>
 #include <gcu/formula.h>
 #include <gcu/loader.h>
 #include <gcu/molecule.h>
@@ -33,6 +35,7 @@
 #include <gcu/residue.h>
 
 #include <goffice/app/module-plugin-defs.h>
+#include <glib/gi18n-lib.h>
 #include <openbabel/chemdrawcdx.h>
 #include <map>
 #include <string>
@@ -40,6 +43,9 @@
 #include <cstdio>
 #include <libintl.h>
 #include <cstring>
+#include <sstream>
+
+#include <libxml/tree.h>
 
 using namespace std;
 using namespace gcu;
@@ -66,6 +72,40 @@ typedef struct {
 static map<guint16, string> Charsets;
 static map<string, guint16> CharsetIDs;
 
+static gint32 ReadInt (GsfInput *input, int size)
+{
+	gint32 res = 0;
+	switch (size) {
+	case 1:
+		gsf_input_read (input, 1, (guint8*) &res);
+		break;
+	case 2:
+		READINT16 (input, res);
+		break;
+	case 4:
+		READINT32 (input, res);
+		break;
+	}
+	return res;
+}
+
+/*static guint32 ReadUInt (GsfInput *input, int size)
+{
+	guint32 res = 0;
+	switch (size) {
+	case 1:
+		gsf_input_read (input, 1, (guint8*) &res);
+		break;
+	case 2:
+		READINT16 (input, res);
+		break;
+	case 4:
+		READINT32 (input, res);
+		break;
+	}
+	return res;
+}*/
+
 class CDXLoader: public gcu::Loader
 {
 public:
@@ -93,9 +133,12 @@ private:
 	size_t bufsize;
 	map<unsigned, CDXFont> fonts;
 	vector <string> colors;
+	guint8 m_TextAlign, m_TextJustify;
 };
 
-CDXLoader::CDXLoader ()
+CDXLoader::CDXLoader ():
+	m_TextAlign (0),
+	m_TextJustify (0)
 {
 	AddMimeType ("chemical/x-cdx");
 }
@@ -200,8 +243,8 @@ bool CDXLoader::Read  (Document *doc, GsfInput *in, char const *mime_type, IOCon
 			}
 			break;
 			case kCDXProp_ColorTable: {
-				colors.push_back ("1 1 1"); // white
-				colors.push_back ("0 0 0"); // black
+				colors.push_back ("red=\"1\" green=\"1\" blue=\"1\""); // white
+				colors.push_back ("red=\"0\" green=\"0\" blue=\"0\""); // black
 				unsigned nb = (size - 2) / 6;
 				if (!READINT16 (in,size) || size != nb)
 					return false;
@@ -209,11 +252,16 @@ bool CDXLoader::Read  (Document *doc, GsfInput *in, char const *mime_type, IOCon
 				for (unsigned i = 0; i < nb; i++) {
 				if (!READINT16 (in,red) || !READINT16 (in,green) || !READINT16 (in,blue))
 					return false;
-					snprintf (buf, bufsize, "%g %g %g", (double) red / 0xffff, (double) green / 0xffff, (double) blue / 0xffff);
+					snprintf (buf, bufsize, "red=\"%g\" green=\"%g\" blue=\"%g\"", (double) red / 0xffff, (double) green / 0xffff, (double) blue / 0xffff);
 					colors.push_back (buf);
 				}
 				break;
 			}
+			case kCDXProp_CaptionJustification: {
+				if (!gsf_input_read (in, 1, &m_TextAlign))
+					return false;
+				break;
+			}
 			default:
 				if (size)
 					result = (gsf_input_read (in, size, (guint8*) buf));
@@ -383,130 +431,211 @@ bool CDXLoader::ReadAtom (GsfInput *in, Object *parent)
 				if (Z == 6) {
 					if (!ReadFragmentText (in, Atom))
 						goto bad_exit;
-				switch (type) {
-				case 0:
-					// Parse the formula.
-					try {
-						Formula form (buf, GCU_FORMULA_PARSE_RESIDUE);
-						if (Doc) {
-							map< string, Object * >::iterator i;
-							Molecule *mol = dynamic_cast <Molecule *> (Doc->GetFirstChild (i));
-							if (Doc->GetChildrenNumber () != 1 || mol == NULL)
-								goto bad_exit;
-							// compare the formula as interpreted with the document contents
-							// TODO: write this code
-						}
-						// now build a molecule from the formula
-						Molecule *mol2 = NULL;
-						if (Doc)
-							mol2 = Molecule::MoleculeFromFormula (Doc, form);
-						bool replace = true;
-						if (mol2) {
-						} else {
-							// check if the formula contains only one atom
-							std::list<FormulaElt *> const &items = form.GetElements ();
-							if (items.size () == 1 && dynamic_cast <FormulaAtom const *> (items.front ()))
-								replace = false;
-						}
-						if (replace) {
-							string pos = Atom->GetProperty (GCU_PROP_POS2D);
-							Molecule *mol = dynamic_cast <Molecule *> (parent);
-							if (mol)
-								mol->Remove (Atom);
-							delete Atom;
-							Atom = Object::CreateObject ("fragment", parent);
-							Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
-							snprintf (buf, bufsize, "a%d", Id);
-							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
-							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
-							Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
-						}
-						if (!Doc)
-							Doc = parent->GetDocument ()->GetApp ()->CreateNewDocument ();
-					}
-					catch (parse_error &error) {
-						return false;
-					}
-					break;
-				case 4: {
-					bool amb;
-					Residue const *res = Residue::GetResidue (buf, &amb);
-					if (res != NULL) {
-						map< string, Object * >::iterator i;
-						Molecule *mol = dynamic_cast <Molecule *> (Doc->GetFirstChild (i));
-						if (mol == NULL)
-							goto bad_exit;
-						if (*res == *mol) {
-							// Residue has been identified to the known one
-							string pos = Atom->GetProperty (GCU_PROP_POS2D);
-							Molecule *mol = dynamic_cast <Molecule *> (parent);
-							if (mol)
-								mol->Remove (Atom);
-							delete Atom;
-							Atom = Object::CreateObject ("fragment", parent);
-							Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
-							snprintf (buf, bufsize, "a%d", Id);
-							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
-							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
-							Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
-						} else {
-							// FIXME: should the document care with the residues?
-						}
-					} else {
-						// FIXME: Unkown residue: add it to the database? or just to the document?
-					}
-					break;
-				}
-				case 5:
-					// First, parse the formula.
-					{
-						map< string, Object * >::iterator i;
-						Molecule *mol = dynamic_cast <Molecule *> (Doc->GetFirstChild (i));
-						// Do the molecule have a pseudo-atom?
-						bool have_pseudo = false;
-						Object *obj = mol->GetFirstChild (i);
-						gcu::Atom *a;
-						while (obj) {
-							a = dynamic_cast <gcu::Atom *> (obj);
-							if (a && ! a->GetZ ()) {
-								have_pseudo = true;
-								break;
-							}
-							obj = mol->GetNextChild (i);
-						}
-						if (mol == NULL)
-							goto bad_exit;
+					switch (type) {
+					case 0:
+						// Parse the formula.
 						try {
-							// First, parse the formula.
 							Formula form (buf, GCU_FORMULA_PARSE_RESIDUE);
-							// now build a molecule from the formula
-							Molecule *mol2 = Molecule::MoleculeFromFormula (Doc, form, have_pseudo);
-							// Now see if it matches with the molecule
-							if (!mol2 || !(*mol == *mol2)) {
-								// try adding a new residue
-								printf("failed for %s\n",buf);
+							if (Doc) {
+								map< string, Object * >::iterator i;
+								Molecule *mol = dynamic_cast <Molecule *> (Doc->GetFirstChild (i));
+								if (Doc->GetChildrenNumber () != 1 || mol == NULL)
+									goto bad_exit;
+								// compare the formula as interpreted with the document contents
+								// TODO: write this code
 							}
+							// now build a molecule from the formula
+							Molecule *mol2 = NULL;
+							if (Doc)
+								mol2 = Molecule::MoleculeFromFormula (Doc, form);
+							bool replace = true;
+							if (mol2) {
+							} else {
+								// check if the formula contains only one atom
+								std::list<FormulaElt *> const &items = form.GetElements ();
+								if (items.size () == 1 && dynamic_cast <FormulaAtom const *> (items.front ()))
+									replace = false;
+							}
+							if (replace) {
+								string pos = Atom->GetProperty (GCU_PROP_POS2D);
+								Molecule *mol = dynamic_cast <Molecule *> (parent);
+								if (mol)
+									mol->Remove (Atom);
+								delete Atom;
+								Atom = Object::CreateObject ("fragment", parent);
+								Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
+								snprintf (buf, bufsize, "a%d", Id);
+								Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+								Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
+								Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
+							}
+							if (!Doc)
+								Doc = parent->GetDocument ()->GetApp ()->CreateNewDocument ();
 						}
 						catch (parse_error &error) {
-							int start, length;
-							puts (error.what (start, length));
+							return false;
 						}
-						string pos = Atom->GetProperty (GCU_PROP_POS2D);
-						mol = dynamic_cast <Molecule *> (parent);
-						if (mol)
-							mol->Remove (Atom);
-						delete Atom;
-						Atom = Object::CreateObject ("fragment", parent);
-						Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
-						snprintf (buf, bufsize, "a%d", Id);
-						Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
-						Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
-						Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
+						break;
+					case 4: {
+						bool amb;
+						Residue const *res = parent->GetDocument ()->GetResidue (buf, &amb);
+						if (res != NULL) {
+							map< string, Object * >::iterator i;
+							Molecule *mol = dynamic_cast <Molecule *> (Doc->GetFirstChild (i));
+							if (mol == NULL)
+								goto bad_exit;
+							if (*res == *mol) {
+								// Residue has been identified to the known one
+								string pos = Atom->GetProperty (GCU_PROP_POS2D);
+								Molecule *mol = dynamic_cast <Molecule *> (parent);
+								if (mol)
+									mol->Remove (Atom);
+								delete Atom;
+								Atom = Object::CreateObject ("fragment", parent);
+								Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
+								snprintf (buf, bufsize, "a%d", Id);
+								Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+								Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
+								Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
+							} else {
+								// FIXME: should the document care with the residues?
+								g_warning (_("Unsupported feature, please report!"));
+							}
+						} else {
+							// FIXME: Unkown residue: add it to the database? or just to the document?
+							g_warning (_("Unsupported feature, please report!"));
+						}
+						break;
+					}
+					case 5:
+						// First, parse the formula.
+						{
+							map< string, Object * >::iterator i;
+							Molecule *mol = dynamic_cast <Molecule *> (Doc->GetFirstChild (i));
+							// Do the molecule have a pseudo-atom?
+							bool have_pseudo = false;
+							Object *obj = mol->GetFirstChild (i);
+							gcu::Atom *a = NULL;
+							while (obj) {
+								a = dynamic_cast <gcu::Atom *> (obj);
+								if (a && ! a->GetZ ()) {
+									have_pseudo = true;
+									break;
+								}
+								obj = mol->GetNextChild (i);
+							}
+							if (mol == NULL)
+								goto bad_exit;
+							try {
+								// First, parse the formula.
+								Formula form (buf, GCU_FORMULA_PARSE_RESIDUE);
+								// now build a molecule from the formula
+								Molecule *mol2 = Molecule::MoleculeFromFormula (Doc, form, have_pseudo);
+								// Now see if it matches with the molecule
+								if (!mol2 || !(*mol == *mol2)) {
+									if (have_pseudo) {
+										// try adding a new residue
+										// first examine the first atom
+										map <gcu::Atom*, gcu::Bond*>::iterator i;
+										gcu::Bond *b = a->GetFirstBond (i);
+										int residue_offset = 0;
+										if (!b)
+											goto fragment_error;
+										gcu::Atom *a2 = b->GetAtom (a);
+										if (!a2)
+											goto fragment_error;
+										list<FormulaElt *> const &elts = form.GetElements ();
+										list<FormulaElt *>::const_iterator j = elts.begin ();
+										FormulaAtom *fatom = dynamic_cast <FormulaAtom *> (*j);
+										int valence;
+										if (!fatom || fatom->elt != a2->GetZ ())
+											goto fragment_add;
+										valence = Element::GetElement (fatom->elt)->GetDefaultValence ();
+										switch (valence) {
+										case 2: {
+											/* remove the first atom and replace it by a pseudo-atom, then add the residue
+											this helps with things begining with an oxygen or a sulfur, but might be 
+											not enough n other cases */
+											double x, y;
+											a2->GetCoords (&x, &y);
+											a->SetCoords (x, y);
+											a->RemoveBond (b);
+											a2->RemoveBond (b);
+											mol->Remove (b);
+											delete b;
+											if (a2->GetBondsNumber () > 1)
+												goto fragment_error;
+											b = a2->GetFirstBond (i);
+											if (b->GetOrder () != 1)
+												goto fragment_error;
+											b->ReplaceAtom (a2, a);
+											a->AddBond (b);
+											mol->Remove (a2);
+											delete a2;
+											// now remove the atom from the new residue symbol
+											residue_offset += fatom->end;
+											break;
+										}
+										case 3:
+											// we do not support that at the moment
+											goto fragment_error;
+											break;
+										default:
+											// we do not support that at the moment
+											goto fragment_error;
+										}
+fragment_add:
+										// Try create a new document, using the symbol as name
+										parent->GetDocument ()->CreateResidue (buf + residue_offset, buf + residue_offset, mol);
+										goto fragment_success;
+									}
+fragment_error:
+									g_warning (_("failed for %s\n"),buf);
+								}
+							}
+							catch (parse_error &error) {
+								int start, length;
+								puts (error.what (start, length));
+							}
+fragment_success:
+							string pos = Atom->GetProperty (GCU_PROP_POS2D);
+							mol = dynamic_cast <Molecule *> (parent);
+							if (mol)
+								mol->Remove (Atom);
+							delete Atom;
+							Atom = Object::CreateObject ("fragment", parent);
+							Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
+							snprintf (buf, bufsize, "a%d", Id);
+							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
+							Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
+						}
+						break;
+					case 7: {
+						bool amb;
+						Residue const *res = parent->GetDocument ()->GetResidue (buf, &amb);
+						if (res != NULL && res->GetGeneric ()) {
+							string pos = Atom->GetProperty (GCU_PROP_POS2D);
+							Molecule *mol = dynamic_cast <Molecule *> (parent);
+							if (mol)
+								mol->Remove (Atom);
+							delete Atom;
+							Atom = Object::CreateObject ("fragment", parent);
+							Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
+							snprintf (buf, bufsize, "a%d", Id);
+							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
+							Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
+						} else {
+							// TODO: import it in the document
+							g_warning (_("Unsupported feature, please report!"));
+						}
+						break;
+					}
+					default:
+						g_warning (_("Unsupported feature, please report!"));
+						break;
 					}
-					break;
-				default:
-					break;
-				}
 					break;
 				}
 			default:
@@ -662,11 +791,20 @@ bool CDXLoader::ReadBond (GsfInput *in, Object *parent)
 	return true;
 }
 
+typedef struct {
+	guint16 index;
+	guint16 font;
+	guint16 face;
+	guint16 size;
+	guint16 color;
+} attribs;
+
 bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 {
 	guint16 code;
 	Object *Text= Object::CreateObject ("text", parent);
 	guint32 Id;
+	guint8 TextAlign = 0xfe, TextJustify = 0xfe;
 	if (!READINT32 (in,Id))
 		return false;
 	snprintf (buf, bufsize, "t%d", Id);
@@ -696,26 +834,179 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 			}
 			case kCDXProp_Text: {
 				guint16 nb;
+				bool interpret = false;
+				attribs attrs, attrs0;
+				attrs0.index = 0; // makes gcc happy
+				attrs0.face = 0; // ditto
 				if (!READINT16 (in,nb))
 					return false;
+				list <attribs> attributes;
 				size -=2;
+				guint16 *n = &attrs.index;
 				for (int i =0; i < nb; i++) {
 					if (size < 10)
 						return false;
-					guint16 n[5];
 					for (int j = 0; j < 5; j++)
 						if (!READINT16 (in,n[j]))
 							return false;
+					attributes.push_back (attrs);
 					size -= 10;
 				}
 				if (size < 1)
 					return false;
-				if (!gsf_input_read (in, size, (guint8*) buf))
-					return false;
-				buf[size] = 0;
-				Text->SetProperty (GCU_PROP_TEXT_TEXT, buf);
+				if (attributes.empty ()) {
+					if (!gsf_input_read (in, size, (guint8*) buf))
+						return false;
+					buf[size] = 0;
+					Text->SetProperty (GCU_PROP_TEXT_TEXT, buf);
+				} else {
+					ostringstream str;
+					str << "<text>";
+					while (!attributes.empty ()) {
+						attrs = attributes.front ();
+						attributes.pop_front ();
+						if (attrs.index > 0) {
+							attrs0.index = attrs.index - attrs0.index;
+							if (!gsf_input_read (in, attrs0.index, (guint8*) buf))
+								return false;
+							buf[attrs0.index] = 0;
+							// supposing the text is ASCII !!
+							if (interpret) {
+								// for now put all numbers as subscripts
+								// FIXME: fix this kludgy code
+								int cur = 0;
+								while (cur < attrs0.index) {
+									while (cur < attrs0.index && (buf[cur] < '0' || buf[cur] > '9')){printf("cur=%d c=%c\n",cur,buf[cur]);
+										str << buf[cur++];}
+									if (cur < attrs0.index) {
+										if (attrs0.face & 4)
+											str << "</u>";
+										if (attrs0.face & 2)
+											str << "</i>";
+										if (attrs0.face & 1)
+											str << "</b>";
+										str << "</fore></font><font name=\"" << fonts[attrs.font].name << " " << (double) attrs.size / 30. << "\">";
+										str << "<fore " << colors[attrs.color] << ">";
+										str << "<sub height=\"" << (double) attrs.size / 60. << "\">";
+										while (buf[cur] >= '0' && buf[cur] <= '9'){printf("cur=%d c=%c\n",cur,buf[cur]);
+											str << buf[cur++];}
+										str << "</sub></fore></font><font name=\"" << fonts[attrs.font].name << " " << (double) attrs.size / 20. << "\">";
+										str << "<fore " << colors[attrs.color] << ">";
+										if (attrs0.face & 1)
+											str << "<b>";
+										if (attrs0.face & 2)
+											str << "<i>";
+										if (attrs0.face & 4)
+											str << "<u>";
+									}
+								}
+							} else
+								str << buf;
+							size -= attrs0.index;
+							if ((attrs0.face & 0x60) == 0x60)
+								interpret = false;
+							else if (attrs0.face & 0x40)
+								str << "</sup>";
+							else if (attrs0.face & 0x20)
+								str << "</sub>";
+							if (attrs0.face & 4)
+								str << "</u>";
+							if (attrs0.face & 2)
+								str << "</i>";
+							if (attrs0.face & 1)
+								str << "</b>";
+							str << "</fore>";
+							str << "</font>";
+						}
+						if ((attrs.face & 0x60) != 0 && (attrs.face & 0x60) != 0x60)
+							attrs.size = attrs.size * 2 / 3;
+						str << "<font name=\"" << fonts[attrs.font].name << " " << (double) attrs.size / 20. << "\">";
+						str << "<fore " << colors[attrs.color] << ">";
+						if (attrs.face & 1)
+							str << "<b>";
+						if (attrs.face & 2)
+							str << "<i>";
+						if (attrs.face & 4)
+							str << "<u>";
+						// skip 0x08 == outline since it is not supported
+						// skip 0x10 == shadow since it is not supported
+						if ((attrs.face & 0x60) == 0x60)
+							interpret = true;
+						else if (attrs.face & 0x20)
+							str << "<sub height=\"" << (double) attrs.size / 40. << "\">";
+						else if (attrs.face & 0x40)
+							str << "<sup height=\"" << (double) attrs.size / 20. << "\">";
+						attrs0 = attrs;
+					}
+					if (!gsf_input_read (in, size, (guint8*) buf))
+						return false;
+					buf[size] = 0;
+					bool opened = true;
+					// supposing the text is ASCII!!
+					if (interpret) {
+						// for now put all numbers as subscripts
+						// FIXME: fix this kludgy code
+						int cur = 0;
+						while (cur < size) {
+							while (cur < size && (buf[cur] < '0' || buf[cur] > '9'))
+								str << buf[cur++];
+							if (cur < size) {
+								if (attrs0.face & 4)
+									str << "</u>";
+								if (attrs0.face & 2)
+									str << "</i>";
+								if (attrs0.face & 1)
+									str << "</b>";
+								str << "</fore></font><font name=\"" << fonts[attrs.font].name << " " << (double) attrs.size / 30. << "\">";
+								str << "<fore " << colors[attrs.color] << ">";
+								str << "<sub height=\"" << (double) attrs.size / 60. << "\">";
+								while (buf[cur] >= '0' && buf[cur] <= '9')
+									str << buf[cur++];
+								str << "</sub></fore></font>";
+								if (cur < size) {
+									str << "<font name=\"" << fonts[attrs.font].name << " " << (double) attrs.size / 20. << "\">";
+									str << "<fore " << colors[attrs.color] << ">";
+									if (attrs0.face & 1)
+										str << "<b>";
+									if (attrs0.face & 4)
+										str << "<u>";
+									if (attrs0.face & 2)
+										str << "<i>";
+								} else
+									opened = false;
+							}
+						}
+					} else
+						str << buf;
+					if (opened) {
+						if ((attrs0.face & 0x60) != 0x60) {
+							if (attrs0.face & 0x40)
+								str << "</sup>";
+							else if (attrs0.face & 0x20)
+								str << "</sub>";
+						}
+						if (attrs0.face & 4)
+							str << "</u>";
+						if (attrs0.face & 2)
+							str << "</i>";
+						if (attrs0.face & 1)
+							str << "</b>";
+						str << "</fore>";
+						str << "</font>";
+						}
+					str << "</text>";
+					Text->SetProperty (GCU_PROP_TEXT_MARKUP, str.str().c_str());
+				}
 				break;
 			}
+			case kCDXProp_Justification:
+				if (!gsf_input_read (in, 1, &TextJustify))
+					return false;
+				break;
+			case kCDXProp_CaptionJustification:
+				if (!gsf_input_read (in, 1, &TextAlign))
+					return false;
+				break;
 			default:
 				if (size && !gsf_input_read (in, size, (guint8*) buf))
 					return false;
@@ -724,6 +1015,41 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 		if (!READINT16 (in,code))
 			return false;
 	}
+	if (TextAlign == 0xfe)
+		TextAlign = m_TextAlign;
+	switch (TextAlign) {
+	case 0xff: 
+		Text->SetProperty (GCU_PROP_TEXT_ALIGNMENT, "right");
+		break;
+	case 0: 
+		Text->SetProperty (GCU_PROP_TEXT_ALIGNMENT, "left");
+		break;
+	case 1: 
+		Text->SetProperty (GCU_PROP_TEXT_ALIGNMENT, "center");
+		break;
+	default:
+	// Other cases are not currently supported
+		break;
+	}
+	if (TextJustify == 0xfe)
+		TextJustify = m_TextJustify;
+	switch (TextJustify) {
+	case 0xff: 
+		Text->SetProperty (GCU_PROP_TEXT_JUSTIFICATION, "right");
+		break;
+	case 0: 
+		Text->SetProperty (GCU_PROP_TEXT_JUSTIFICATION, "left");
+		break;
+	case 1: 
+		Text->SetProperty (GCU_PROP_TEXT_JUSTIFICATION, "center");
+		break;
+	case 2: 
+		Text->SetProperty (GCU_PROP_TEXT_JUSTIFICATION, "justify");
+		break;
+	default:
+	// Other cases are not currently supported
+		break;
+	}
 	return true;
 }
 
@@ -789,12 +1115,10 @@ bool CDXLoader::ReadGraphic  (GsfInput *in, Object *parent)
 					return false;
 				break;
 			case kCDXProp_Graphic_Type:
-				if (size != 2 || !READINT16 (in,type))
-					return false;
+				type = ReadInt (in, size);
 				break;
 			case kCDXProp_Arrow_Type:
-				if (size != 2 || !READINT16 (in,arrow_type))
-					return false;
+				arrow_type = ReadInt (in, size);
 				break;
 			default:
 				if (size && !gsf_input_read (in, size, (guint8*) buf))
